@@ -18,7 +18,7 @@
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #                                                                               #
 # This script will compile and run the c examples from source files installed   #
-# in /usr/pkg/share/examples/hdf5/c using h5cc or h5pc.  The order for running                 #
+# in /usr/local/share/examples/hdf5/c using h5cc or h5pc.  The order for running                 #
 # programs with RunTest in the MAIN section below is taken from the Makefile.   #
 # The order is important since some of the test programs use data files created #
 # by earlier test programs.  Any future additions should be placed accordingly. #
@@ -42,8 +42,8 @@ EXIT_FAILURE=1
 # $() won't clobbered it.
 #
 prefix_relto_examplesdir=$(
-prefix=/usr/pkg
-examplesdir=/usr/pkg/share/examples/hdf5
+prefix=/usr/local
+examplesdir=/usr/local/share/examples/hdf5
 if [ ${examplesdir##${prefix}/} != ${examplesdir} ]; then
 	echo $(echo ${examplesdir##${prefix}/} | \
 	    sed 's,[^/][^/]*,..,g')
